<?php

App::uses('AppBehavior', 'Model/Behavior');

class CalculateTaxBehavior extends AppBehavior {

	/**
	 * link the model that has this behavior with the tax levels model
	 * so we can contain data
	 */
	protected function _linkModels(Model $Model) {
		$Model->bindModel(
			array(
				'belongsTo' => array(
					'TaxLevel' => array(
						'className' => 'EvTax.TaxLevel'
					)
				)
			),
			false
		);
	}

	/**
	 * Initialise the behavior - store the settings against the model
	 *
	 * @see ModelBehavior::setup()
	 */
	public function setup(Model $Model, $settings = array()) {
		$defaults = array(
			'fields' => array(),
			'TaxRateModel' => false,
			'TaxRateModelId' => false
		);

		$this->settings[$Model->alias] = array_merge(
			$defaults,
			$settings
		);

		if (empty($this->settings[$Model->alias]['TaxRateModel'])) {
			$this->_linkModels($Model);
		}
	}

	/**
	 * beforeFind can be used to cancel find operations, or modify the query that will be executed.
	 * By returning null/false you can abort a find. By returning an array you can modify/replace the query
	 * that is going to be run.
	 *
	 * @param Model $model Model using this behavior
	 * @param array $query Data used to execute this query, i.e. conditions, order, etc.
	 * @return bool|array False or null will abort the operation. You can return an array to replace the
	 *   $query that will be eventually run.
	 */
	public function beforeFind(Model $Model, $query) {
		if (empty($this->settings[$Model->alias]['TaxRateModel'])) {
			if (! isset($query['contain']) || (! isset($query['contain']['TaxLevel']) && array_search('TaxLevel', $query['contain']) === false)) {
				$query['contain'][] = 'TaxLevel';
			}
		}

		return $query;
	}

	/**
	 * After find callback. Can be used to modify any results returned by find.
	 *
	 * @param Model $model Model using this behavior
	 * @param mixed $results The results of the find operation
	 * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
	 * @return mixed An array value will replace the value of $results - any other value will be ignored.
	 */
	public function afterFind(Model $Model, $results, $primary = false) {
		if (empty($this->settings[$Model->alias]['fields']) || ! is_array($this->settings[$Model->alias]['fields'])) {
			return $results;
		}

		if (! empty($this->settings[$Model->alias]['TaxRateModel'])) {

			$TaxRateModel = EvClassRegistry::init($this->settings[$Model->alias]['TaxRateModel']);
		}

		foreach ($results as $key => $item) {

			if (! empty($item['TaxLevel']['rate'])) {
				$rate = $item['TaxLevel']['rate'];
			} elseif (isset($TaxRateModel)) {
				$rate = $this->_getRate($Model, $TaxRateModel, $item);
			} else {
				$rate = 0.00;
			}

			foreach ($this->settings[$Model->alias]['fields'] as $field) {
				if (! array_key_exists($field, $item[$Model->alias])) {
					continue;
				}

				$results[$key][$Model->alias][$field . '_incTax'] = $item[$Model->alias][$field] + ($item[$Model->alias][$field] * ($rate / 100));
			}
		}

		return $results;
	}

	/**
	 * work out the tax rate
	 * - either query table
	 * - retrieve from stored array
	 *
	 * @param 	object 			Model object of the original model that called this behavior
	 * @param 	object 			Model Object of the related model that has the tax rate id
	 * @param 	array 			data array
	 * @return 	decimal 		decimal rate, will return zero if none found
	 */
	protected function _getRate($Model, $TaxRateModel, $item) {
		$modelId = Hash::get($item, $Model->alias . '.' . $this->settings[$Model->alias]['TaxRateModelId']);

		if (! empty($this->settings[$Model->alias]['taxRateCache'][$modelId])) {
			return $this->settings[$Model->alias]['taxRateCache'][$modelId];
		}

		if (! empty($modelId)) {
			$rate = $TaxRateModel->getTaxRate($modelId);

			if (! empty($rate)) {
				$this->settings[$Model->alias]['taxRateCache'][$modelId] = $rate;
				return $rate;
			}
		}

		return intval(0);
	}
}
